#include "avrmain.h"
#include "theqube.h"

void sent_output(uint8_t row_out,uint16_t col_out);
int  load_animation();
void save_animation();


Frame frames_data[200] = {
  0x00000001UL,
  0x00000002UL,
  0x00000004UL,
  0x00000008UL,
  0x00000010UL,
  0x00000020UL,
  0x00000040UL,
  0x00000080UL,
  0x00000100UL,
  0x00000200UL,
  0x00000400UL,
  0x00000800UL,
  0x00001000UL,
  0x00002000UL,
  0x00004000UL,
  0x00008000UL,
  0x00010000UL,
  0x00020000UL,
  0x00040000UL,
  0x00080000UL,
  0x00100000UL,
  0x00200000UL,
  0x00400000UL,
  0x00800000UL,
  0x01000000UL,
  0x02000000UL,
  0x04000000UL,
};
Animation animation = {frames_data,27,0,DEFAULT_REFRESH_RATE,0,sent_output};


main(){
#ifdef __AVR__
  DDRC = DDRB = 0x3F;
  usbInit();
  PORTC = 0b111111;
  usbDeviceDisconnect();
  PORTB = 0b111000; _delay_ms(450);
  usbDeviceConnect();
  PORTB = 0b111111; _delay_ms(150);
  sei();
  PORTB = 0b111110; _delay_ms(150);
  PORTB = 0b111101; _delay_ms(150);
  PORTB = 0b111011; _delay_ms(150);
  PORTC = PORTB=0 ; _delay_ms(200);
  if(load_animation())
    save_animation(); 
#endif
  while ( 1 )
  {
#ifdef __AVR__
    usbPoll();
#endif
    UpdateAnimation(&animation);
  }
}




void sent_output(uint8_t row_out,uint16_t col_out){
#ifdef __AVR__
  _delay_us(600);
  PORTC = PORTB = 0;    // clear
  _delay_us(600);
  PORTB  = ( row_out & 0x007 );
  PORTC  = ( col_out & 0x03F );
  PORTB |= ( col_out & 0x1C0 ) >> 3;
#else
  __PCTEST__
#endif
}



#ifdef __AVR__
int load_animation()
{
  uint16_t eeprom_total_frames = eeprom_read_word((uint16_t*)0x0000);
  if( eeprom_total_frames == 0xFFFF )
    return 1;
  animation.total_frames = eeprom_total_frames;
  eeprom_read_block((uint32_t*)frames_data,(uint32_t*)0x0004,animation.total_frames*sizeof(Frame));
  return 0;
}
void save_animation()
{
  uint16_t eeprom_total_frames = ((animation.total_frames>(508/sizeof(Frame)))?(508/sizeof(Frame)):animation.total_frames);
  eeprom_write_word((uint16_t*)0x0000,eeprom_total_frames);
  eeprom_write_block((uint32_t*)frames_data,(uint32_t*)0x0004,eeprom_total_frames*sizeof(Frame));
}




uint8_t usb_frame_stored;
usbMsgLen_t usbFunctionSetup(uint8_t data[8]){
  usbRequest_t *rq = (void *)data;
  switch (rq->bRequest) {
    case VENDOR_RQ_SET_FRAME_DATA:
      usb_frame_stored = 0;
      animation.current_frame = 0;
      animation.total_frames = rq->wValue.bytes[0];
      return USB_NO_MSG;
    case VENDOR_RQ_SAVE_ANIMATION:
      save_animation();
      break;
  }
  return 0;
}
uchar usbFunctionWrite(uchar *data, uchar len){
  if ( len < sizeof(Frame) )
    animation.total_frames = usb_frame_stored;
  else
  {
    frames_data[usb_frame_stored++] = *((Frame *)data);
    
    if ( len-sizeof(Frame) < sizeof(Frame) )
      animation.total_frames = usb_frame_stored;
    else if( usb_frame_stored < animation.total_frames )
      frames_data[usb_frame_stored++] = *((Frame *)(&data[4]));
  }
  
  return ( usb_frame_stored >= animation.total_frames );
}
#endif



