#include <iostream>
#include <fstream>
#include <string>
#include <stdint.h>
#include "usbclass.h"

using namespace std;

Frame frame_data[200];
uint8_t total_frames = 0;
USBClass usb;

bool connectUSB(){
  int x;
  string cmd;
  x = usb.Connect(0x16c0, 0x05dc);
  while( x < 0 )
  {
    cout << "USB connection error(" << x <<")" << endl;
    while (true)
    {
      cout << "Retry?" << endl << ": ";
      cin >> cmd;
      if( cmd == "yes" || cmd == "y" )
      {
        usb.Disconnect();
        x = usb.Connect(0x16c0, 0x05dc);
        break;
      }
      else if( cmd == "no" || cmd == "n" )
        return 0;
      else
        cout << "Please answer yes/no" << endl;
    }
  }
  cout << "USB connected" << endl;
  return 1;
}

int load(const char *file){
  int i,j;
  total_frames = 0;
  fstream fs(file);
  if (fs.is_open())
  {
    while (!fs.eof())
    {
      string bf;
      Frame fbf = 0;
      for ( i=0 ; i<3 ; i++ )
      {
        fs >> bf;
        if ( bf.length() != 9 )
          break;
        for ( j = 0 ; j < 9 ; j++)
          fbf |= (bf[j] == '1')<<(j+i*9);
      }
      if( i==3 )
        frame_data[total_frames++] = fbf;
      else
        break;
    }
    return 1;
  }
  return 0;
}

int main(int argc,char *argv[])
{
  int argn = 1;
  int x;
  cout << "TheQube Commander" << endl;
  string cmd;
  if(connectUSB())
  {
    while (true) {
      if (argn<argc)
        cmd = argv[argn++];
      else
      {
        cout << ": ";
        cin >> cmd;
      }

      if ( cmd == "send" )
      {
        if (argn<argc)
          cmd = argv[argn++];
        else
          cin >> cmd;
        if(load(cmd.c_str()))
        {
          cout << (uint32_t)total_frames << " frames loaded" << endl;
          x = usb.SendFrameData(total_frames,frame_data);
          if( x < 0 )
            cout << "Error sending data(" << x << ")" << endl;
          else if ( x > 0 )
            cout << x << " bytes of data has been sent" << endl;
        }
        else
          cout << "File '" << cmd << "' couldn't be open" << endl;
      }
      else if ( cmd == "save" )
      {
        usb.SaveAnimation();
        cout << "Animation saved to EEPROM" << endl;
      }
      else if ( cmd == "show" )
      {
        for (int i=0; i<total_frames; i++)
          cout << frame_data[i] << endl;
      }
      else if ( cmd == "reconnect" )
      {
        usb.Disconnect();
        cout << "Reconnecting..." << endl;
        connectUSB();
      }
      else if ( cmd == "exit" || cmd == "quit" )
      {
        cout << "Good bye" << endl;
        break;
      }
      else
        cout << "'" << cmd << "' : Unknow command" << endl;
      cin.clear();
    }
  }
  return 0;
}